subroutine CalcEnergy(Pos, Walls, L, rc, NA, PEnergy, Dim, NAtom, NWall)
    implicit none
    integer, intent(in) :: NA, Dim, NAtom, NWall
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
	real(8), intent(in), dimension(0:NWall-1, 0:Dim-1) :: Walls
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
	real(8), dimension(Dim) :: rij, Posi, riw
	real(8) ::  d2, id2, id6, id12, LJshift0,LJshift1,LJshift2,LJshift3, dw2, rct, atr,rep
	integer :: i, j, w
	atr = 1.
	rep = 1.
	!precompute Lennard-Jones shift value to save time
	LJshift0 = 4. * (rc**(-12)-rc**(-6)) 
	LJshift1 = 4. * (4.*rc**(-12)-rc**(-6)) 
	LJshift2 = 4. * (rc**(-12)-2.*rc**(-6)) 
	LJshift3 = 4. * (rep*rc**(-12)-atr*rc**(-6))
	PEnergy = 0.
	rct = rc * rc
 
	do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
		
		!calc (wall-particle) interactions
		do w = 0, NWall - 1
			riw = Walls(w,:) - Posi
			riw(:2) = riw(:2) - L * dnint(riw(:2)/L) !min image in x,y dir
			dw2 = sum(riw*riw)
			if (dw2 > rct) then
				cycle
			end if	
			id2 = 1. / dw2	   		 !inverse squared distance
			id6 = id2 * id2 * id2    !inverse sixth distance
			id12 = id6 * id6  
			if (i < NA) then
				!Wall - A interaction      
				PEnergy = PEnergy + 4.*(4.*id12-id6) - LJshift1
			else 
				!Wall - B interaction      
				PEnergy = PEnergy + 4.*(id12-2.*id6) - LJshift2
			end if
		enddo 
		
	  !calc particle-particle interactions
       do j = i + 1, NAtom - 1
			!this is nonbonded interaction
			rij = Pos(j,:) - Posi
			rij(:2) = rij(:2) - L * dnint(rij(:2)/L)
			d2 = sum(rij * rij)      !avoid computation of square root
			if (d2 > rct) then
				cycle
			end if	
			id2 = 1. / d2            !inverse squared distance
			id6 = id2 * id2 * id2    !inverse sixth distance
			id12 = id6 * id6         !inverse twelvth distance
			if (j < NA .and. i < NA) then
				!this is nonbonded A-A interaction  
				PEnergy = PEnergy + 4. * (id12 - id6) - LJshift0  !cut and shift
			else if (j >= NA .and. i >= NA) then
				!this is nonbonded B-B interaction 
				PEnergy = PEnergy + 4.*(id12- id6) - LJshift0
			else if (j < NA .and. i >= NA) then
				!this is nonbonded A-B interaction
				PEnergy = PEnergy + 4.*(rep*id12-atr*id6) - LJshift3
			else if (j >= NA .and. i < NA) then
				!this is nonbonded B-A interaction
				PEnergy = PEnergy + 4.*(rep*id12-atr*id6) - LJshift3
			else
				write(*,*) "error"
			end if
		enddo
	enddo
end subroutine

subroutine CalcEnergyForces(Pos, Walls, L, rc, NA, PEnergy, Forces, Dim, NAtom, NWall)
    implicit none
    integer, intent(in) :: NA, Dim, NAtom, NWall
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
	real(8), intent(in), dimension(0:NWall-1, 0:Dim-1) :: Walls
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
	integer :: i, j, w
	real(8), dimension(Dim) :: rij, Fij, Posi,riw, Fiw
	real(8) :: d2, id2, id6, id12, LJshift0,LJshift1,LJshift2,LJshift3, dw2, rct, atr,rep
	atr = 1.
	rep = 1.
	PEnergy = 0.
	Forces = 0.
	LJshift0 = 4. * (rc**(-12)-rc**(-6)) !precompute Lennard-Jones shift value to save time
	LJshift1 = 4. * (4.*rc**(-12)-rc**(-6)) 
	LJshift2 = 4. * (rc**(-12)-2.*rc**(-6)) 
	LJshift3 = 4. * (rep*rc**(-12)-atr*rc**(-6)) 
	rct = rc * rc
	
	 do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)

		!calc (wall-particle) interactions
		do w = 0, NWall - 1
			riw = Walls(w,:) - Posi
			riw(:2) = riw(:2) - L * dnint(riw(:2) / L) !min image in x,y dir
			dw2 = sum(riw*riw)
			if (dw2 > rct) then
				cycle
			end if
			id2 = 1. / dw2	   		 !inverse squared distance
			id6 = id2 * id2 * id2    !inverse sixth distance
			id12 = id6 * id6  
			if ( i < NA ) then
				!Wall - A interactions       
				PEnergy = PEnergy + 4.*(4.*id12-id6) - LJshift1
				Fiw = riw * ((-48. * 4.*id12 + 24. * id6) * id2)
				Forces(i,:) = Forces(i,:) + Fiw
			else
				!Wall - B interaction      
				PEnergy = PEnergy + 4.*(id12-2.*id6) - LJshift2
				Fiw = riw * ((-48. * id12 + 24. * 2.*id6) * id2)
				Forces(i,:) = Forces(i,:) + Fiw
			end if
		enddo 
		
		!calc particle-particle interactions
        do j = i + 1, NAtom - 1
				!this is nonbonded interaction
				rij = Pos(j,:) - Posi
				rij(:2) = rij(:2) - L * dnint(rij(:2) /L)!determine minimum image distance
				d2 = sum(rij * rij)          !avoid computation of square root
				if (d2 > rct) then
					cycle
				end if
				id2 = 1. / d2            !inverse squared distance
				id6 = id2 * id2 * id2    !inverse sixth distance
				id12 = id6 * id6         !inverse twelvth distance
				if (j < NA .and. i < NA) then
					!this is nonbonded A-A interaction  
					PEnergy = PEnergy + 4. * (id12 - id6) - LJshift0  !cut and shift
					Fij = rij * ((-48. * id12 + 24. * id6) * id2)
					Forces(i,:) = Forces(i,:) + Fij
					Forces(j,:) = Forces(j,:) - Fij	
				else if (j >= NA .and. i >= NA) then
					!this is nonbonded B-B interaction 
					PEnergy = PEnergy + 4.*(id12-id6) - LJshift0
					Fij = rij * ((-48. * id12 + 24. * id6) * id2)
					Forces(i,:) = Forces(i,:) + Fij
					Forces(j,:) = Forces(j,:) - Fij	
				else if (j < NA .and. i >= NA) then
					!this is nonbonded A-B interaction
					PEnergy = PEnergy + 4.*(rep*id12-atr*id6) - LJshift3
					Fij = rij * ((-48. * rep*id12 + 24. *atr* id6) * id2)
					Forces(i,:) = Forces(i,:) + Fij
					Forces(j,:) = Forces(j,:) - Fij	
				else if (j >= NA .and. i < NA) then
					!this is nonbonded B-A interaction
					PEnergy = PEnergy + 4.*(rep*id12-atr*id6) - LJshift3
					Fij = rij * ((-48. *rep*id12 + 24.*atr*id6) * id2)
					Forces(i,:) = Forces(i,:) + Fij
					Forces(j,:) = Forces(j,:) - Fij	
				else
					write(*,*) "error"
				end if
		enddo
	enddo	
end subroutine

subroutine VVIntegrate(Pos, Walls, Vel, Accel, L, rc, NA, dt, KEnergy, PEnergy, Dim, NAtom, NWall)
    implicit none
    integer, intent(in) :: NA, Dim, NAtom, NWall
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
	real(8), intent(in), dimension(0:NWall-1, 0:Dim-1) :: Walls
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, Walls, L, rc, NA, PEnergy, Accel, Dim, NAtom, NWall)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine